var messages = {
    "ACK": {
        "desc": "General acknowledgment message",
        "name": "ACK",
        "segments": {
            "desc": "General acknowledgment message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADR_A19": {
        "desc": "Patient query",
        "name": "ADR_A19",
        "segments": {
            "desc": "Patient query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QUERY_RESPONSE",
                    "desc": "Query response",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "EVN",
                            "desc": "Event type",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NK1",
                            "desc": "Next of kin",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "DB1",
                            "desc": "Disability segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DRG",
                            "desc": "Diagnosis related group",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PROCEDURE",
                            "desc": "Procedure",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PR1",
                                    "desc": "Procedures",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "ACC",
                            "desc": "Accident",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "UB1",
                            "desc": "Ub82  data",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "UB2",
                            "desc": "Ub92 data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A01": {
        "desc": "Admit/visit notification",
        "name": "ADT_A01",
        "segments": {
            "desc": "Admit/visit notification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A02": {
        "desc": "Transfer a patient",
        "name": "ADT_A02",
        "segments": {
            "desc": "Transfer a patient",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A03": {
        "desc": "Discharge/end visit",
        "name": "ADT_A03",
        "segments": {
            "desc": "Discharge/end visit",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A04": {
        "desc": "Register a patient",
        "name": "ADT_A04",
        "segments": {
            "desc": "Register a patient",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A05": {
        "desc": "Pre-admit a patient",
        "name": "ADT_A05",
        "segments": {
            "desc": "Pre-admit a patient",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A06": {
        "desc": "Change an outpatient to an inpatient",
        "name": "ADT_A06",
        "segments": {
            "desc": "Change an outpatient to an inpatient",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A07": {
        "desc": "Change an inpatient to an outpatient",
        "name": "ADT_A07",
        "segments": {
            "desc": "Change an inpatient to an outpatient",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A08": {
        "desc": "Update patient information",
        "name": "ADT_A08",
        "segments": {
            "desc": "Update patient information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A09": {
        "desc": "Patient departing - tracking",
        "name": "ADT_A09",
        "segments": {
            "desc": "Patient departing - tracking",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A10": {
        "desc": "Patient arriving - tracking",
        "name": "ADT_A10",
        "segments": {
            "desc": "Patient arriving - tracking",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A11": {
        "desc": "Cancel admit/visit notification",
        "name": "ADT_A11",
        "segments": {
            "desc": "Cancel admit/visit notification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A12": {
        "desc": "Cancel transfer",
        "name": "ADT_A12",
        "segments": {
            "desc": "Cancel transfer",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A13": {
        "desc": "Cancel discharge/end visit",
        "name": "ADT_A13",
        "segments": {
            "desc": "Cancel discharge/end visit",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A14": {
        "desc": "Pending admit",
        "name": "ADT_A14",
        "segments": {
            "desc": "Pending admit",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A15": {
        "desc": "Pending transfer",
        "name": "ADT_A15",
        "segments": {
            "desc": "Pending transfer",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A16": {
        "desc": "Pending discharge",
        "name": "ADT_A16",
        "segments": {
            "desc": "Pending discharge",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A17": {
        "desc": "Swap patients",
        "name": "ADT_A17",
        "segments": {
            "desc": "Swap patients",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A18": {
        "desc": "Merge patient information",
        "name": "ADT_A18",
        "segments": {
            "desc": "Merge patient information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A20": {
        "desc": "Bed status update",
        "name": "ADT_A20",
        "segments": {
            "desc": "Bed status update",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NPU",
                    "desc": "Bed status update",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A21": {
        "desc": "Patient goes on a \"leave of absence\"",
        "name": "ADT_A21",
        "segments": {
            "desc": "Patient goes on a \"leave of absence\"",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A22": {
        "desc": "Patient returns from a \"leave of absence\"",
        "name": "ADT_A22",
        "segments": {
            "desc": "Patient returns from a \"leave of absence\"",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A23": {
        "desc": "Delete a patient record",
        "name": "ADT_A23",
        "segments": {
            "desc": "Delete a patient record",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A24": {
        "desc": "Link patient information",
        "name": "ADT_A24",
        "segments": {
            "desc": "Link patient information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A25": {
        "desc": "Cancel pending discharge",
        "name": "ADT_A25",
        "segments": {
            "desc": "Cancel pending discharge",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A26": {
        "desc": "Cancel pending transfer",
        "name": "ADT_A26",
        "segments": {
            "desc": "Cancel pending transfer",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A27": {
        "desc": "Cancel pending admit",
        "name": "ADT_A27",
        "segments": {
            "desc": "Cancel pending admit",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A28": {
        "desc": "Add person information",
        "name": "ADT_A28",
        "segments": {
            "desc": "Add person information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A29": {
        "desc": "Delete person information",
        "name": "ADT_A29",
        "segments": {
            "desc": "Delete person information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A30": {
        "desc": "Merge person information",
        "name": "ADT_A30",
        "segments": {
            "desc": "Merge person information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A31": {
        "desc": "Update person information",
        "name": "ADT_A31",
        "segments": {
            "desc": "Update person information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ROL",
                            "desc": "Role",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB1",
                    "desc": "Ub82  data",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "UB2",
                    "desc": "Ub92 data",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A32": {
        "desc": "Cancel patient arriving - tracking",
        "name": "ADT_A32",
        "segments": {
            "desc": "Cancel patient arriving - tracking",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A33": {
        "desc": "Cancel patient departing - tracking",
        "name": "ADT_A33",
        "segments": {
            "desc": "Cancel patient departing - tracking",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A34": {
        "desc": "Merge patient information - patient id only",
        "name": "ADT_A34",
        "segments": {
            "desc": "Merge patient information - patient id only",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A35": {
        "desc": "Merge patient information - account number only",
        "name": "ADT_A35",
        "segments": {
            "desc": "Merge patient information - account number only",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A36": {
        "desc": "Merge patient information - patient id and account number",
        "name": "ADT_A36",
        "segments": {
            "desc": "Merge patient information - patient id and account number",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A37": {
        "desc": "Unlink patient information",
        "name": "ADT_A37",
        "segments": {
            "desc": "Unlink patient information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A38": {
        "desc": "Cancel pre-admit",
        "name": "ADT_A38",
        "segments": {
            "desc": "Cancel pre-admit",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A39": {
        "desc": "Merge person - patient id",
        "name": "ADT_A39",
        "segments": {
            "desc": "Merge person - patient id",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "MRG",
                            "desc": "Merge patient information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "ADT_A40": {
        "desc": "Merge patient - patient identifier list",
        "name": "ADT_A40",
        "segments": {
            "desc": "Merge patient - patient identifier list",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "MRG",
                            "desc": "Merge patient information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "ADT_A41": {
        "desc": "Merge account - patient account number",
        "name": "ADT_A41",
        "segments": {
            "desc": "Merge account - patient account number",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "MRG",
                            "desc": "Merge patient information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "ADT_A42": {
        "desc": "Merge visit - visit number",
        "name": "ADT_A42",
        "segments": {
            "desc": "Merge visit - visit number",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "MRG",
                            "desc": "Merge patient information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "ADT_A43": {
        "desc": "Move patient information - patient identifier list",
        "name": "ADT_A43",
        "segments": {
            "desc": "Move patient information - patient identifier list",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "MRG",
                            "desc": "Merge patient information",
                            "min": 1,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "ADT_A44": {
        "desc": "Move account information - patient account number",
        "name": "ADT_A44",
        "segments": {
            "desc": "Move account information - patient account number",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "MRG",
                            "desc": "Merge patient information",
                            "min": 1,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "ADT_A45": {
        "desc": "Move visit information - visit number",
        "name": "ADT_A45",
        "segments": {
            "desc": "Move visit information - visit number",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MERGE_INFO",
                    "desc": "Merge info",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MRG",
                            "desc": "Merge patient information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "ADT_A46": {
        "desc": "Change patient id (for backward compatibility only)",
        "name": "ADT_A46",
        "segments": {
            "desc": "Change patient id (for backward compatibility only)",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A47": {
        "desc": "Change patient identifier list",
        "name": "ADT_A47",
        "segments": {
            "desc": "Change patient identifier list",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A48": {
        "desc": "Change alternate patient id",
        "name": "ADT_A48",
        "segments": {
            "desc": "Change alternate patient id",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A49": {
        "desc": "Change patient account number",
        "name": "ADT_A49",
        "segments": {
            "desc": "Change patient account number",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A50": {
        "desc": "Change visit number",
        "name": "ADT_A50",
        "segments": {
            "desc": "Change visit number",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A51": {
        "desc": "Change alternate visit id",
        "name": "ADT_A51",
        "segments": {
            "desc": "Change alternate visit id",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MRG",
                    "desc": "Merge patient information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A53": {
        "desc": "Cancel patient returns from a leave of absence",
        "name": "ADT_A53",
        "segments": {
            "desc": "Cancel patient returns from a leave of absence",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A54": {
        "desc": "Change attending doctor",
        "name": "ADT_A54",
        "segments": {
            "desc": "Change attending doctor",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "ROL",
                    "desc": "Role",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "ROL",
                    "desc": "Role",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A55": {
        "desc": "Cancel change attending doctor",
        "name": "ADT_A55",
        "segments": {
            "desc": "Cancel change attending doctor",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A60": {
        "desc": "Update allergy information",
        "name": "ADT_A60",
        "segments": {
            "desc": "Update allergy information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "IAM",
                    "desc": "Patient adverse reaction information - unique iden",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "ADT_A61": {
        "desc": "Change consulting doctor",
        "name": "ADT_A61",
        "segments": {
            "desc": "Change consulting doctor",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ROL",
                    "desc": "Role",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ADT_A62": {
        "desc": "Cancel change consulting doctor",
        "name": "ADT_A62",
        "segments": {
            "desc": "Cancel change consulting doctor",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ROL",
                    "desc": "Role",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "BAR_P01": {
        "desc": "Add patient accounts",
        "name": "BAR_P01",
        "segments": {
            "desc": "Add patient accounts",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "DB1",
                            "desc": "Disability segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DRG",
                            "desc": "Diagnosis related group",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PROCEDURE",
                            "desc": "Procedure",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PR1",
                                    "desc": "Procedures",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "NK1",
                            "desc": "Next of kin",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "ACC",
                            "desc": "Accident",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "UB1",
                            "desc": "Ub82  data",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "UB2",
                            "desc": "Ub92 data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "BAR_P02": {
        "desc": "Purge patient accounts",
        "name": "BAR_P02",
        "segments": {
            "desc": "Purge patient accounts",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "DB1",
                            "desc": "Disability segment",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "BAR_P05": {
        "desc": "Update account",
        "name": "BAR_P05",
        "segments": {
            "desc": "Update account",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "DB1",
                            "desc": "Disability segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DRG",
                            "desc": "Diagnosis related group",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PROCEDURE",
                            "desc": "Procedure",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PR1",
                                    "desc": "Procedures",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "NK1",
                            "desc": "Next of kin",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "ACC",
                            "desc": "Accident",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "UB1",
                            "desc": "Ub82  data",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "UB2",
                            "desc": "Ub92 data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "BAR_P06": {
        "desc": "End account",
        "name": "BAR_P06",
        "segments": {
            "desc": "End account",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "CRM_C01": {
        "desc": "Register a patient on a clinical trial",
        "name": "CRM_C01",
        "segments": {
            "desc": "Register a patient on a clinical trial",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CSP",
                            "desc": "Clinical study phase",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "CRM_C02": {
        "desc": "Cancel a patient registration on clinical trial",
        "name": "CRM_C02",
        "segments": {
            "desc": "Cancel a patient registration on clinical trial",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CSP",
                            "desc": "Clinical study phase",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "CRM_C03": {
        "desc": "Correct/update registration information",
        "name": "CRM_C03",
        "segments": {
            "desc": "Correct/update registration information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CSP",
                            "desc": "Clinical study phase",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "CRM_C04": {
        "desc": "Patient has gone off a clinical trial",
        "name": "CRM_C04",
        "segments": {
            "desc": "Patient has gone off a clinical trial",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CSP",
                            "desc": "Clinical study phase",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "CRM_C05": {
        "desc": "Patient enters phase of clinical trial",
        "name": "CRM_C05",
        "segments": {
            "desc": "Patient enters phase of clinical trial",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CSP",
                            "desc": "Clinical study phase",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "CRM_C06": {
        "desc": "Cancel patient entering a phase",
        "name": "CRM_C06",
        "segments": {
            "desc": "Cancel patient entering a phase",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CSP",
                            "desc": "Clinical study phase",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "CRM_C07": {
        "desc": "Correct/update phase information",
        "name": "CRM_C07",
        "segments": {
            "desc": "Correct/update phase information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CSP",
                            "desc": "Clinical study phase",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "CRM_C08": {
        "desc": "Patient has gone off phase of clinical trial",
        "name": "CRM_C08",
        "segments": {
            "desc": "Patient has gone off phase of clinical trial",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CSP",
                            "desc": "Clinical study phase",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "CSU_C09": {
        "desc": "Automated time intervals for reporting, like monthly",
        "name": "CSU_C09",
        "segments": {
            "desc": "Automated time intervals for reporting, like monthly",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VISIT",
                            "desc": "Visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "STUDY_PHASE",
                            "desc": "Study phase",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "CSP",
                                    "desc": "Clinical study phase",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "STUDY_SCHEDULE",
                                    "desc": "Study schedule",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "CSS",
                                            "desc": "Clinical study data schedule",
                                            "min": 0,
                                            "max": 1
                                        },
                                        {
                                            "name": "STUDY_OBSERVATION",
                                            "desc": "Study observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 0,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "STUDY_PHARM",
                                            "desc": "Study pharm",
                                            "min": 1,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 0,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "RX_ADMIN",
                                                    "desc": "Rx admin",
                                                    "min": 1,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "RXA",
                                                            "desc": "Pharmacy administration segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "RXR",
                                                            "desc": "Pharmacy route segment",
                                                            "min": 1,
                                                            "max": 1
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "CSU_C10": {
        "desc": "Patient completes the clinical trial",
        "name": "CSU_C10",
        "segments": {
            "desc": "Patient completes the clinical trial",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VISIT",
                            "desc": "Visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "STUDY_PHASE",
                            "desc": "Study phase",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "CSP",
                                    "desc": "Clinical study phase",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "STUDY_SCHEDULE",
                                    "desc": "Study schedule",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "CSS",
                                            "desc": "Clinical study data schedule",
                                            "min": 0,
                                            "max": 1
                                        },
                                        {
                                            "name": "STUDY_OBSERVATION",
                                            "desc": "Study observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 0,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "STUDY_PHARM",
                                            "desc": "Study pharm",
                                            "min": 1,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 0,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "RX_ADMIN",
                                                    "desc": "Rx admin",
                                                    "min": 1,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "RXA",
                                                            "desc": "Pharmacy administration segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "RXR",
                                                            "desc": "Pharmacy route segment",
                                                            "min": 1,
                                                            "max": 1
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "CSU_C11": {
        "desc": "Patient completes a phase of the clinical trial",
        "name": "CSU_C11",
        "segments": {
            "desc": "Patient completes a phase of the clinical trial",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VISIT",
                            "desc": "Visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "STUDY_PHASE",
                            "desc": "Study phase",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "CSP",
                                    "desc": "Clinical study phase",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "STUDY_SCHEDULE",
                                    "desc": "Study schedule",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "CSS",
                                            "desc": "Clinical study data schedule",
                                            "min": 0,
                                            "max": 1
                                        },
                                        {
                                            "name": "STUDY_OBSERVATION",
                                            "desc": "Study observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 0,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "STUDY_PHARM",
                                            "desc": "Study pharm",
                                            "min": 1,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 0,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "RX_ADMIN",
                                                    "desc": "Rx admin",
                                                    "min": 1,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "RXA",
                                                            "desc": "Pharmacy administration segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "RXR",
                                                            "desc": "Pharmacy route segment",
                                                            "min": 1,
                                                            "max": 1
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "CSU_C12": {
        "desc": "Update/correction of patient order/result information",
        "name": "CSU_C12",
        "segments": {
            "desc": "Update/correction of patient order/result information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VISIT",
                            "desc": "Visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "CSR",
                            "desc": "Clinical study registration",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "STUDY_PHASE",
                            "desc": "Study phase",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "CSP",
                                    "desc": "Clinical study phase",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "STUDY_SCHEDULE",
                                    "desc": "Study schedule",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "CSS",
                                            "desc": "Clinical study data schedule",
                                            "min": 0,
                                            "max": 1
                                        },
                                        {
                                            "name": "STUDY_OBSERVATION",
                                            "desc": "Study observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 0,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "STUDY_PHARM",
                                            "desc": "Study pharm",
                                            "min": 1,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 0,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "RX_ADMIN",
                                                    "desc": "Rx admin",
                                                    "min": 1,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "RXA",
                                                            "desc": "Pharmacy administration segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "RXR",
                                                            "desc": "Pharmacy route segment",
                                                            "min": 1,
                                                            "max": 1
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "DFT_P03": {
        "desc": "Post detail financial transaction",
        "name": "DFT_P03",
        "segments": {
            "desc": "Post detail financial transaction",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PV2",
                    "desc": "Patient visit - additional information",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DB1",
                    "desc": "Disability segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "FINANCIAL",
                    "desc": "Financial",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "FT1",
                            "desc": "Financial transaction",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "FINANCIAL_PROCEDURE",
                            "desc": "Financial procedure",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PR1",
                                    "desc": "Procedures",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "DOC_T12": {
        "desc": "Document response",
        "name": "DOC_T12",
        "segments": {
            "desc": "Document response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RESULT",
                    "desc": "Result",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "EVN",
                            "desc": "Event type",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "TXA",
                            "desc": "Document notification segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "DSR_Q01": {
        "desc": "Display response message",
        "name": "DSR_Q01",
        "segments": {
            "desc": "Display response message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QAK",
                    "desc": "Query acknowledgement",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DSP",
                    "desc": "Display data segment",
                    "min": 1,
                    "max": 0
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "DSR_Q03": {
        "desc": "Deferred response to a query",
        "name": "DSR_Q03",
        "segments": {
            "desc": "Deferred response to a query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DSP",
                    "desc": "Display data segment",
                    "min": 1,
                    "max": 0
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "EDR_Q01": {
        "desc": "Enhanced display response",
        "name": "EDR_Q01",
        "segments": {
            "desc": "Enhanced display response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QAK",
                    "desc": "Query acknowledgement",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "DSP",
                    "desc": "Display data segment",
                    "min": 1,
                    "max": 0
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "EQQ_Q01": {
        "desc": "Embedded query language query",
        "name": "EQQ_Q01",
        "segments": {
            "desc": "Embedded query language query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EQL",
                    "desc": "Embedded query language",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ERP_Q01": {
        "desc": "Erp message",
        "name": "ERP_Q01",
        "segments": {
            "desc": "Erp message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QAK",
                    "desc": "Query acknowledgement",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERQ",
                    "desc": "Event replay query segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "MDM_T01": {
        "desc": "Original document notification",
        "name": "MDM_T01",
        "segments": {
            "desc": "Original document notification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "MDM_T02": {
        "desc": "Original document notification and content",
        "name": "MDM_T02",
        "segments": {
            "desc": "Original document notification and content",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 1,
                    "max": 0
                }
            ]
        }
    },
    "MDM_T03": {
        "desc": "Document status change notification",
        "name": "MDM_T03",
        "segments": {
            "desc": "Document status change notification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "MDM_T04": {
        "desc": "Document status change notification and content",
        "name": "MDM_T04",
        "segments": {
            "desc": "Document status change notification and content",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 1,
                    "max": 0
                }
            ]
        }
    },
    "MDM_T05": {
        "desc": "Document addendum notification",
        "name": "MDM_T05",
        "segments": {
            "desc": "Document addendum notification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "MDM_T06": {
        "desc": "Document addendum notification and content",
        "name": "MDM_T06",
        "segments": {
            "desc": "Document addendum notification and content",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 1,
                    "max": 0
                }
            ]
        }
    },
    "MDM_T07": {
        "desc": "Document edit notification",
        "name": "MDM_T07",
        "segments": {
            "desc": "Document edit notification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "MDM_T08": {
        "desc": "Document edit notification and content",
        "name": "MDM_T08",
        "segments": {
            "desc": "Document edit notification and content",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 1,
                    "max": 0
                }
            ]
        }
    },
    "MDM_T09": {
        "desc": "Document replacement notification",
        "name": "MDM_T09",
        "segments": {
            "desc": "Document replacement notification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "MDM_T10": {
        "desc": "Document replacement notification and content",
        "name": "MDM_T10",
        "segments": {
            "desc": "Document replacement notification and content",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "OBX",
                    "desc": "Observation segment",
                    "min": 1,
                    "max": 0
                }
            ]
        }
    },
    "MDM_T11": {
        "desc": "Document cancel notification",
        "name": "MDM_T11",
        "segments": {
            "desc": "Document cancel notification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PV1",
                    "desc": "Patient visit",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "TXA",
                    "desc": "Document notification segment",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "MFK_M01": {
        "desc": "Master file application acknowledgment",
        "name": "MFK_M01",
        "segments": {
            "desc": "Master file application acknowledgment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MFA",
                    "desc": "Master file acknowledgement segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF",
                    "desc": "Mf",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFA",
                            "desc": "Master file acknowledgement segment",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "MFK_M02": {
        "desc": "Master file application acknowledgment",
        "name": "MFK_M02",
        "segments": {
            "desc": "Master file application acknowledgment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFA",
                    "desc": "Master file acknowledgement segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "MFK_M03": {
        "desc": "Master file application acknowledgment",
        "name": "MFK_M03",
        "segments": {
            "desc": "Master file application acknowledgment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFA",
                    "desc": "Master file acknowledgement segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "MFK_M04": {
        "desc": "Master file application acknowledgment",
        "name": "MFK_M04",
        "segments": {
            "desc": "Master file application acknowledgment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFA",
                    "desc": "Master file acknowledgement segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "MFK_M05": {
        "desc": "Master file application acknowledgment",
        "name": "MFK_M05",
        "segments": {
            "desc": "Master file application acknowledgment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFA",
                    "desc": "Master file acknowledgement segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "MFK_M07": {
        "desc": "Master file acknowledgment",
        "name": "MFK_M07",
        "segments": {
            "desc": "Master file acknowledgment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFA",
                    "desc": "Master file acknowledgement segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "MFN_M01": {
        "desc": "Master file not otherwise specified",
        "name": "MFN_M01",
        "segments": {
            "desc": "Master file not otherwise specified",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF",
                    "desc": "Mf",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFE",
                            "desc": "Master file entry segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "GenericSegment",
                            "desc": "Generic segment",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "MFN_M02": {
        "desc": "Master file - staff practitioner",
        "name": "MFN_M02",
        "segments": {
            "desc": "Master file - staff practitioner",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF_STAFF",
                    "desc": "Mf staff",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFE",
                            "desc": "Master file entry segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "STF",
                            "desc": "Staff identification segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PRA",
                            "desc": "Practitioner detail segment",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "MFN_M04": {
        "desc": "Master files charge description",
        "name": "MFN_M04",
        "segments": {
            "desc": "Master files charge description",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF_CDM",
                    "desc": "Mf cdm",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFE",
                            "desc": "Master file entry segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CDM",
                            "desc": "Charge description master",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PRC",
                            "desc": "Pricing",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "MFN_M05": {
        "desc": "Patient location master file",
        "name": "MFN_M05",
        "segments": {
            "desc": "Patient location master file",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF_LOCATION",
                    "desc": "Mf location",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFE",
                            "desc": "Master file entry segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "LOC",
                            "desc": "Location identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "LCH",
                            "desc": "Location characteristic",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "LRL",
                            "desc": "Location relationship",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "MF_LOC_DEPT",
                            "desc": "Mf loc dept",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "LDP",
                                    "desc": "Location department",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "LCH",
                                    "desc": "Location characteristic",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "LCC",
                                    "desc": "Location charge code",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "MFN_M06": {
        "desc": "Clinical study with phases and schedules master file",
        "name": "MFN_M06",
        "segments": {
            "desc": "Clinical study with phases and schedules master file",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF_CLIN_STUDY",
                    "desc": "Mf clin study",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFE",
                            "desc": "Master file entry segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CM0",
                            "desc": "Clinical study master",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "MF_PHASE_SCHED_DETAIL",
                            "desc": "Mf phase sched detail",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "CM1",
                                    "desc": "Clinical study phase master",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CM2",
                                    "desc": "Clinical study schedule master",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "MFN_M07": {
        "desc": "Clinical study without phases but with schedules master file",
        "name": "MFN_M07",
        "segments": {
            "desc": "Clinical study without phases but with schedules master file",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF_CLIN_STUDY",
                    "desc": "Mf clin study",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFE",
                            "desc": "Master file entry segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CM0",
                            "desc": "Clinical study master",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "MF_PHASE_SCHED_DETAIL",
                            "desc": "Mf phase sched detail",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "CM1",
                                    "desc": "Clinical study phase master",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CM2",
                                    "desc": "Clinical study schedule master",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "MFN_M08": {
        "desc": "Test/observation (numeric) master file",
        "name": "MFN_M08",
        "segments": {
            "desc": "Test/observation (numeric) master file",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF_TEST_NUMERIC",
                    "desc": "Mf test numeric",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFE",
                            "desc": "Master file entry segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "OM1",
                            "desc": "General - fields that apply to most observations",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "XX",
                            "desc": "Xx",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "OM2",
                                    "desc": "Numeric observation",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "OM3",
                                    "desc": "Categorical test/observation",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "OM4",
                                    "desc": "Observations that require specimens",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "MFN_M09": {
        "desc": "Test/observation (categorical) master file",
        "name": "MFN_M09",
        "segments": {
            "desc": "Test/observation (categorical) master file",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF_TEST_CATEGORICAL",
                    "desc": "Mf test categorical",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFE",
                            "desc": "Master file entry segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "MF_TEST_CAT_DETAIL",
                            "desc": "Mf test cat detail",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "OM3",
                                    "desc": "Categorical test/observation",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "OM4",
                                    "desc": "Observations that require specimens",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "MFN_M10": {
        "desc": "Test/observation batteries master file",
        "name": "MFN_M10",
        "segments": {
            "desc": "Test/observation batteries master file",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF_TEST_BATTERIES",
                    "desc": "Mf test batteries",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OM5",
                            "desc": "Observation batteries",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "OM4",
                            "desc": "Observations that require specimens",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "MFN_M11": {
        "desc": "Test/calculated observations master file",
        "name": "MFN_M11",
        "segments": {
            "desc": "Test/calculated observations master file",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MFI",
                    "desc": "Master file identification segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MF_TEST_CALCULATED",
                    "desc": "Mf test calculated",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "MFE",
                            "desc": "Master file entry segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "OM1",
                            "desc": "General - fields that apply to most observations",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "MF_TEST_CALC_DETAIL",
                            "desc": "Mf test calc detail",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "OM6",
                                    "desc": "Observations that are calculated from other observations",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "OM2",
                                    "desc": "Numeric observation",
                                    "min": 1,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "OMD_O01": {
        "desc": "Dietary order",
        "name": "OMD_O01",
        "segments": {
            "desc": "Dietary order",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "ORDER_DIET",
                    "desc": "Order diet",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "DIET",
                            "desc": "Diet",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "ODS",
                                    "desc": "Dietary orders, supplements, and preferences",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "OBSERVATION",
                                    "desc": "Observation",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "ORDER_TRAY",
                    "desc": "Order tray",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ODT",
                            "desc": "Diet tray instructions segment",
                            "min": 1,
                            "max": 0
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "OMN_O01": {
        "desc": "Non-stock requisition order message",
        "name": "OMN_O01",
        "segments": {
            "desc": "Non-stock requisition order message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "ORDER",
                    "desc": "Order",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ORDER_DETAIL",
                            "desc": "Order detail",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "RQD",
                                    "desc": "Requisition detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "RQ1",
                                    "desc": "Requisition detail-1 segment",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "OBSERVATION",
                                    "desc": "Observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "BLG",
                            "desc": "Billing segment",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "OMS_O01": {
        "desc": "Stock requisition order message",
        "name": "OMS_O01",
        "segments": {
            "desc": "Stock requisition order message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "ORDER",
                    "desc": "Order",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ORDER_DETAIL",
                            "desc": "Order detail",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "RQD",
                                    "desc": "Requisition detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "OBSERVATION",
                                    "desc": "Observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "BLG",
                            "desc": "Billing segment",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "ORD_O02": {
        "desc": "Dietary order acknowledgment message",
        "name": "ORD_O02",
        "segments": {
            "desc": "Dietary order acknowledgment message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER_DIET",
                            "desc": "Order diet",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ODS",
                                    "desc": "Dietary orders, supplements, and preferences",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER_TRAY",
                            "desc": "Order tray",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ODT",
                                    "desc": "Diet tray instructions segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "ORF_R02": {
        "desc": "Orf message",
        "name": "ORF_R02",
        "segments": {
            "desc": "Orf message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QUERY_RESPONSE",
                    "desc": "Query response",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "OBR",
                                    "desc": "Observation request segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "OBSERVATION",
                                    "desc": "Observation",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 0,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "CTI",
                                    "desc": "Clinical trial identification",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ORF_R04": {
        "desc": "Response to query; transmission of requested observation",
        "name": "ORF_R04",
        "segments": {
            "desc": "Response to query; transmission of requested observation",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QUERY_RESPONSE",
                    "desc": "Query response",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "OBR",
                                    "desc": "Observation request segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "OBSERVATION",
                                    "desc": "Observation",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 0,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "CTI",
                                    "desc": "Clinical trial identification",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ORM_O01": {
        "desc": "Order message",
        "name": "ORM_O01",
        "segments": {
            "desc": "Order message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "ORDER",
                    "desc": "Order",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ORDER_DETAIL",
                            "desc": "Order detail",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "OBR,RQD,RQ1,RXO,ODS,ODT",
                                    "desc": "Details",
                                    "min": 0,
                                    "max": 0,
                                    "compounds": [
                                        {
                                            "name": "OBR",
                                            "desc": "Observation request",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RQD",
                                            "desc": "Requisition detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RQ1",
                                            "desc": "Requisition detail-1",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXO",
                                            "desc": "Pharmacy/treatment order",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ODS",
                                            "desc": "Dietary orders, supplements, and preferences",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ODT",
                                            "desc": "Diet tray instructions",
                                            "min": 1,
                                            "max": 1
                                        }
                                    ]
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "OBSERVATION",
                                    "desc": "Observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "CTI",
                            "desc": "Clinical trial identification",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "BLG",
                            "desc": "Billing segment",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "ORN_O02": {
        "desc": "Non-stock requisition - general order acknowledgment message",
        "name": "ORN_O02",
        "segments": {
            "desc": "Non-stock requisition - general order acknowledgment message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "RQD",
                                    "desc": "Requisition detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "RQ1",
                                    "desc": "Requisition detail-1 segment",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "ORR_O02": {
        "desc": "Order response",
        "name": "ORR_O02",
        "segments": {
            "desc": "Order response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "OBR,RQD,RQ1,RXO,ODS,ODT",
                                    "desc": "Details",
                                    "min": 0,
                                    "max": 0,
                                    "compounds": [
                                        {
                                            "name": "OBR",
                                            "desc": "Observation request",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RQD",
                                            "desc": "Requisition detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RQ1",
                                            "desc": "Requisition detail-1",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXO",
                                            "desc": "Pharmacy/treatment order",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ODS",
                                            "desc": "Dietary orders, supplements, and preferences",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ODT",
                                            "desc": "Diet tray instructions",
                                            "min": 1,
                                            "max": 1
                                        }
                                    ]
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "CTI",
                                    "desc": "Clinical trial identification",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "ORU": {
        "desc": "Unsolicited transmission of an observation message",
        "name": "ORU",
        "segments": {
            "desc": "Unsolicited transmission of an observation message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PD1",
                                    "desc": "Patient demographic",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VISIT",
                                    "desc": "Visit",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "PV1",
                                            "desc": "Patient visit",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "PV2",
                                            "desc": "Patient visit - additional information",
                                            "min": 0,
                                            "max": 1
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ORDER_OBSERVATION",
                            "desc": "Order observation",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "OBR",
                                    "desc": "Observation request segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "OBSERVATION",
                                    "desc": "Observation",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 0,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "CTI",
                                    "desc": "Clinical trial identification",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "ORU_R01": {
        "desc": "Unsolicited transmission of an observation message",
        "name": "ORU_R01",
        "segments": {
            "desc": "Unsolicited transmission of an observation message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PD1",
                                    "desc": "Patient demographic",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VISIT",
                                    "desc": "Visit",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "PV1",
                                            "desc": "Patient visit",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "PV2",
                                            "desc": "Patient visit - additional information",
                                            "min": 0,
                                            "max": 1
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ORDER_OBSERVATION",
                            "desc": "Order observation",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "OBR",
                                    "desc": "Observation request segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "OBSERVATION",
                                    "desc": "Observation",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 0,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "CTI",
                                    "desc": "Clinical trial identification",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "OSQ_Q06": {
        "desc": "Query for order status",
        "name": "OSQ_Q06",
        "segments": {
            "desc": "Query for order status",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "OSR_Q06": {
        "desc": "Query response for order status",
        "name": "OSR_Q06",
        "segments": {
            "desc": "Query response for order status",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "OBR",
                                    "desc": "Observation request segment",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "CTI",
                                    "desc": "Clinical trial identification",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "PEX_P07": {
        "desc": "Unsolicited initial individual product experience report",
        "name": "PEX_P07",
        "segments": {
            "desc": "Unsolicited initial individual product experience report",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "EXPERIENCE",
                    "desc": "Experience",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PES",
                            "desc": "Product experience sender",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PEX_OBSERVATION",
                            "desc": "Pex observation",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PEO",
                                    "desc": "Product experience observation",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PEX_CAUSE",
                                    "desc": "Pex cause",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "PCR",
                                            "desc": "Possible causal relationship",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RX_ORDER",
                                            "desc": "Rx order",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "RXE",
                                                    "desc": "Pharmacy encoded order segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "RXR",
                                                    "desc": "Pharmacy route segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "RX_ADMINISTRATION",
                                            "desc": "Rx administration",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "RXA",
                                                    "desc": "Pharmacy administration segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "RXR",
                                                    "desc": "Pharmacy route segment",
                                                    "min": 0,
                                                    "max": 1
                                                }
                                            ]
                                        },
                                        {
                                            "name": "PRB",
                                            "desc": "Problem detail",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "ASSOCIATED_PERSON",
                                            "desc": "Associated person",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "NK1",
                                                    "desc": "Next of kin",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "ASSOCIATED_RX_ORDER",
                                                    "desc": "Associated rx order",
                                                    "min": 0,
                                                    "max": 1,
                                                    "children": [
                                                        {
                                                            "name": "RXE",
                                                            "desc": "Pharmacy encoded order segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "RXR",
                                                            "desc": "Pharmacy route segment",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                },
                                                {
                                                    "name": "ASSOCIATED_RX_ADMIN",
                                                    "desc": "Associated rx admin",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "RXA",
                                                            "desc": "Pharmacy administration segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "RXR",
                                                            "desc": "Pharmacy route segment",
                                                            "min": 0,
                                                            "max": 1
                                                        }
                                                    ]
                                                },
                                                {
                                                    "name": "PRB",
                                                    "desc": "Problem detail",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "STUDY",
                                            "desc": "Study",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "CSR",
                                                    "desc": "Clinical study registration",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "CSP",
                                                    "desc": "Clinical study phase",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PEX_P08": {
        "desc": "Unsolicited update individual product experience report",
        "name": "PEX_P08",
        "segments": {
            "desc": "Unsolicited update individual product experience report",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "EVN",
                    "desc": "Event type",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "EXPERIENCE",
                    "desc": "Experience",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PES",
                            "desc": "Product experience sender",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PEX_OBSERVATION",
                            "desc": "Pex observation",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PEO",
                                    "desc": "Product experience observation",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PEX_CAUSE",
                                    "desc": "Pex cause",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "PCR",
                                            "desc": "Possible causal relationship",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RX_ORDER",
                                            "desc": "Rx order",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "RXE",
                                                    "desc": "Pharmacy encoded order segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "RXR",
                                                    "desc": "Pharmacy route segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "RX_ADMINISTRATION",
                                            "desc": "Rx administration",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "RXA",
                                                    "desc": "Pharmacy administration segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "RXR",
                                                    "desc": "Pharmacy route segment",
                                                    "min": 0,
                                                    "max": 1
                                                }
                                            ]
                                        },
                                        {
                                            "name": "PRB",
                                            "desc": "Problem detail",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "ASSOCIATED_PERSON",
                                            "desc": "Associated person",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "NK1",
                                                    "desc": "Next of kin",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "ASSOCIATED_RX_ORDER",
                                                    "desc": "Associated rx order",
                                                    "min": 0,
                                                    "max": 1,
                                                    "children": [
                                                        {
                                                            "name": "RXE",
                                                            "desc": "Pharmacy encoded order segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "RXR",
                                                            "desc": "Pharmacy route segment",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                },
                                                {
                                                    "name": "ASSOCIATED_RX_ADMIN",
                                                    "desc": "Associated rx admin",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "RXA",
                                                            "desc": "Pharmacy administration segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "RXR",
                                                            "desc": "Pharmacy route segment",
                                                            "min": 0,
                                                            "max": 1
                                                        }
                                                    ]
                                                },
                                                {
                                                    "name": "PRB",
                                                    "desc": "Problem detail",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "STUDY",
                                            "desc": "Study",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "CSR",
                                                    "desc": "Clinical study registration",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "CSP",
                                                    "desc": "Clinical study phase",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PGL_PC6": {
        "desc": "Goal add",
        "name": "PGL_PC6",
        "segments": {
            "desc": "Goal add",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "GOAL",
                    "desc": "Goal",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "GOL",
                            "desc": "Goal detail",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "GOAL_ROLE",
                            "desc": "Goal role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PATHWAY",
                            "desc": "Pathway",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PTH",
                                    "desc": "Pathway",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM",
                            "desc": "Problem",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PRB",
                                    "desc": "Problem detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "PROBLEM_ROLE",
                                    "desc": "Problem role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM_OBSERVATION",
                                    "desc": "Problem observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL",
                                    "desc": "Order detail",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "OBR",
                                            "desc": "Observation request segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "ORDER_OBSERVATION",
                                            "desc": "Order observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PGL_PC7": {
        "desc": "Goal update",
        "name": "PGL_PC7",
        "segments": {
            "desc": "Goal update",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "GOAL",
                    "desc": "Goal",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "GOL",
                            "desc": "Goal detail",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "GOAL_ROLE",
                            "desc": "Goal role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PATHWAY",
                            "desc": "Pathway",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PTH",
                                    "desc": "Pathway",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM",
                            "desc": "Problem",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PRB",
                                    "desc": "Problem detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "PROBLEM_ROLE",
                                    "desc": "Problem role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM_OBSERVATION",
                                    "desc": "Problem observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL",
                                    "desc": "Order detail",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "OBR",
                                            "desc": "Observation request segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "ORDER_OBSERVATION",
                                            "desc": "Order observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PGL_PC8": {
        "desc": "Goal delete",
        "name": "PGL_PC8",
        "segments": {
            "desc": "Goal delete",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "GOAL",
                    "desc": "Goal",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "GOL",
                            "desc": "Goal detail",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "GOAL_ROLE",
                            "desc": "Goal role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PATHWAY",
                            "desc": "Pathway",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PTH",
                                    "desc": "Pathway",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM",
                            "desc": "Problem",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PRB",
                                    "desc": "Problem detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "PROBLEM_ROLE",
                                    "desc": "Problem role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM_OBSERVATION",
                                    "desc": "Problem observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL",
                                    "desc": "Order detail",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "OBR",
                                            "desc": "Observation request segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "ORDER_OBSERVATION",
                                            "desc": "Order observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PIN_I07": {
        "desc": "Unsolicited insurance information",
        "name": "PIN_I07",
        "segments": {
            "desc": "Unsolicited insurance information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "PPG_PCG": {
        "desc": "Pathway (goal-oriented) add",
        "name": "PPG_PCG",
        "segments": {
            "desc": "Pathway (goal-oriented) add",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PATHWAY",
                    "desc": "Pathway",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PTH",
                            "desc": "Pathway",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATHWAY_ROLE",
                            "desc": "Pathway role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GOAL",
                            "desc": "Goal",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "GOL",
                                    "desc": "Goal detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "GOAL_ROLE",
                                    "desc": "Goal role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL_OBSERVATION",
                                    "desc": "Goal observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM",
                                    "desc": "Problem",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "PRB",
                                            "desc": "Problem detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "PROBLEM_ROLE",
                                            "desc": "Problem role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "PROBLEM_OBSERVATION",
                                            "desc": "Problem observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                },
                                {
                                    "name": "ORDER",
                                    "desc": "Order",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ORC",
                                            "desc": "Common order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ORDER_DETAIL",
                                            "desc": "Order detail",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "ORDER_OBSERVATION",
                                                    "desc": "Order observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPG_PCH": {
        "desc": "Pathway (goal-oriented) update",
        "name": "PPG_PCH",
        "segments": {
            "desc": "Pathway (goal-oriented) update",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PATHWAY",
                    "desc": "Pathway",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PTH",
                            "desc": "Pathway",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATHWAY_ROLE",
                            "desc": "Pathway role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GOAL",
                            "desc": "Goal",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "GOL",
                                    "desc": "Goal detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "GOAL_ROLE",
                                    "desc": "Goal role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL_OBSERVATION",
                                    "desc": "Goal observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM",
                                    "desc": "Problem",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "PRB",
                                            "desc": "Problem detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "PROBLEM_ROLE",
                                            "desc": "Problem role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "PROBLEM_OBSERVATION",
                                            "desc": "Problem observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                },
                                {
                                    "name": "ORDER",
                                    "desc": "Order",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ORC",
                                            "desc": "Common order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ORDER_DETAIL",
                                            "desc": "Order detail",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "ORDER_OBSERVATION",
                                                    "desc": "Order observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPG_PCJ": {
        "desc": "Pathway (goal-oriented) delete",
        "name": "PPG_PCJ",
        "segments": {
            "desc": "Pathway (goal-oriented) delete",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PATHWAY",
                    "desc": "Pathway",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PTH",
                            "desc": "Pathway",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATHWAY_ROLE",
                            "desc": "Pathway role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GOAL",
                            "desc": "Goal",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "GOL",
                                    "desc": "Goal detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "GOAL_ROLE",
                                    "desc": "Goal role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL_OBSERVATION",
                                    "desc": "Goal observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM",
                                    "desc": "Problem",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "PRB",
                                            "desc": "Problem detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "PROBLEM_ROLE",
                                            "desc": "Problem role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "PROBLEM_OBSERVATION",
                                            "desc": "Problem observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                },
                                {
                                    "name": "ORDER",
                                    "desc": "Order",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ORC",
                                            "desc": "Common order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ORDER_DETAIL",
                                            "desc": "Order detail",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "ORDER_OBSERVATION",
                                                    "desc": "Order observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPP_PCB": {
        "desc": "Pathway (problem-oriented) add",
        "name": "PPP_PCB",
        "segments": {
            "desc": "Pathway (problem-oriented) add",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PATHWAY",
                    "desc": "Pathway",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PTH",
                            "desc": "Pathway",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATHWAY_ROLE",
                            "desc": "Pathway role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM",
                            "desc": "Problem",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PRB",
                                    "desc": "Problem detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "PROBLEM_ROLE",
                                    "desc": "Problem role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM_OBSERVATION",
                                    "desc": "Problem observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL",
                                    "desc": "Goal",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "GOL",
                                            "desc": "Goal detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "GOAL_ROLE",
                                            "desc": "Goal role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "GOAL_OBSERVATION",
                                            "desc": "Goal observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                },
                                {
                                    "name": "ORDER",
                                    "desc": "Order",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ORC",
                                            "desc": "Common order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ORDER_DETAIL",
                                            "desc": "Order detail",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "ORDER_OBSERVATION",
                                                    "desc": "Order observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPP_PCC": {
        "desc": "Pathway (problem-oriented) update",
        "name": "PPP_PCC",
        "segments": {
            "desc": "Pathway (problem-oriented) update",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PATHWAY",
                    "desc": "Pathway",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PTH",
                            "desc": "Pathway",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATHWAY_ROLE",
                            "desc": "Pathway role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM",
                            "desc": "Problem",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PRB",
                                    "desc": "Problem detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "PROBLEM_ROLE",
                                    "desc": "Problem role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM_OBSERVATION",
                                    "desc": "Problem observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL",
                                    "desc": "Goal",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "GOL",
                                            "desc": "Goal detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "GOAL_ROLE",
                                            "desc": "Goal role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "GOAL_OBSERVATION",
                                            "desc": "Goal observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                },
                                {
                                    "name": "ORDER",
                                    "desc": "Order",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ORC",
                                            "desc": "Common order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ORDER_DETAIL",
                                            "desc": "Order detail",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "ORDER_OBSERVATION",
                                                    "desc": "Order observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPP_PCD": {
        "desc": "Pathway (problem-oriented) delete",
        "name": "PPP_PCD",
        "segments": {
            "desc": "Pathway (problem-oriented) delete",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PATHWAY",
                    "desc": "Pathway",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PTH",
                            "desc": "Pathway",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATHWAY_ROLE",
                            "desc": "Pathway role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM",
                            "desc": "Problem",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PRB",
                                    "desc": "Problem detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "PROBLEM_ROLE",
                                    "desc": "Problem role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM_OBSERVATION",
                                    "desc": "Problem observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL",
                                    "desc": "Goal",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "GOL",
                                            "desc": "Goal detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "GOAL_ROLE",
                                            "desc": "Goal role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "GOAL_OBSERVATION",
                                            "desc": "Goal observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                },
                                {
                                    "name": "ORDER",
                                    "desc": "Order",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ORC",
                                            "desc": "Common order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ORDER_DETAIL",
                                            "desc": "Order detail",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "ORDER_OBSERVATION",
                                                    "desc": "Order observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPR_PC1": {
        "desc": "Problem add",
        "name": "PPR_PC1",
        "segments": {
            "desc": "Problem add",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROBLEM",
                    "desc": "Problem",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRB",
                            "desc": "Problem detail",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PROBLEM_ROLE",
                            "desc": "Problem role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PATHWAY",
                            "desc": "Pathway",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PTH",
                                    "desc": "Pathway",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM_OBSERVATION",
                            "desc": "Problem observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GOAL",
                            "desc": "Goal",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "GOL",
                                    "desc": "Goal detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "GOAL_ROLE",
                                    "desc": "Goal role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL_OBSERVATION",
                                    "desc": "Goal observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL",
                                    "desc": "Order detail",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "OBR",
                                            "desc": "Observation request segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "ORDER_OBSERVATION",
                                            "desc": "Order observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPR_PC2": {
        "desc": "Problem update",
        "name": "PPR_PC2",
        "segments": {
            "desc": "Problem update",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROBLEM",
                    "desc": "Problem",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRB",
                            "desc": "Problem detail",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PROBLEM_ROLE",
                            "desc": "Problem role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PATHWAY",
                            "desc": "Pathway",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PTH",
                                    "desc": "Pathway",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM_OBSERVATION",
                            "desc": "Problem observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GOAL",
                            "desc": "Goal",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "GOL",
                                    "desc": "Goal detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "GOAL_ROLE",
                                    "desc": "Goal role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL_OBSERVATION",
                                    "desc": "Goal observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL",
                                    "desc": "Order detail",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "OBR",
                                            "desc": "Observation request segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "ORDER_OBSERVATION",
                                            "desc": "Order observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPR_PC3": {
        "desc": "Problem delete",
        "name": "PPR_PC3",
        "segments": {
            "desc": "Problem delete",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROBLEM",
                    "desc": "Problem",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRB",
                            "desc": "Problem detail",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "VAR",
                            "desc": "Variance",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PROBLEM_ROLE",
                            "desc": "Problem role",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ROL",
                                    "desc": "Role",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PATHWAY",
                            "desc": "Pathway",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PTH",
                                    "desc": "Pathway",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM_OBSERVATION",
                            "desc": "Problem observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GOAL",
                            "desc": "Goal",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "GOL",
                                    "desc": "Goal detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "GOAL_ROLE",
                                    "desc": "Goal role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL_OBSERVATION",
                                    "desc": "Goal observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL",
                                    "desc": "Order detail",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "OBR",
                                            "desc": "Observation request segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "ORDER_OBSERVATION",
                                            "desc": "Order observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPT_PCL": {
        "desc": "Pathway (goal-oriented) query response",
        "name": "PPT_PCL",
        "segments": {
            "desc": "Pathway (goal-oriented) query response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "PATHWAY",
                            "desc": "Pathway",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PTH",
                                    "desc": "Pathway",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "PATHWAY_ROLE",
                                    "desc": "Pathway role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL",
                                    "desc": "Goal",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "GOL",
                                            "desc": "Goal detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "GOAL_ROLE",
                                            "desc": "Goal role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "GOAL_OBSERVATION",
                                            "desc": "Goal observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "PROBLEM",
                                            "desc": "Problem",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "PRB",
                                                    "desc": "Problem detail",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "PROBLEM_ROLE",
                                                    "desc": "Problem role",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "ROL",
                                                            "desc": "Role",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                },
                                                {
                                                    "name": "PROBLEM_OBSERVATION",
                                                    "desc": "Problem observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        },
                                        {
                                            "name": "ORDER",
                                            "desc": "Order",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "ORDER_DETAIL",
                                                    "desc": "Order detail",
                                                    "min": 0,
                                                    "max": 1,
                                                    "children": [
                                                        {
                                                            "name": "OBR",
                                                            "desc": "Observation request segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "ORDER_OBSERVATION",
                                                            "desc": "Order observation",
                                                            "min": 0,
                                                            "max": 0,
                                                            "children": [
                                                                {
                                                                    "name": "OBX",
                                                                    "desc": "Observation segment",
                                                                    "min": 1,
                                                                    "max": 1
                                                                },
                                                                {
                                                                    "name": "NTE",
                                                                    "desc": "Notes and comments segment",
                                                                    "min": 0,
                                                                    "max": 0
                                                                },
                                                                {
                                                                    "name": "VAR",
                                                                    "desc": "Variance",
                                                                    "min": 0,
                                                                    "max": 0
                                                                }
                                                            ]
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PPV_PCA": {
        "desc": "Goal response",
        "name": "PPV_PCA",
        "segments": {
            "desc": "Goal response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "GOAL",
                            "desc": "Goal",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "GOL",
                                    "desc": "Goal detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "GOAL_ROLE",
                                    "desc": "Goal role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL_PATHWAY",
                                    "desc": "Goal pathway",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "PTH",
                                            "desc": "Pathway",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL_OBSERVATION",
                                    "desc": "Goal observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM",
                                    "desc": "Problem",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "PRB",
                                            "desc": "Problem detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "PROBLEM_ROLE",
                                            "desc": "Problem role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "PROBLEM_OBSERVATION",
                                            "desc": "Problem observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                },
                                {
                                    "name": "ORDER",
                                    "desc": "Order",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ORC",
                                            "desc": "Common order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ORDER_DETAIL",
                                            "desc": "Order detail",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "ORDER_OBSERVATION",
                                                    "desc": "Order observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PRR_PC5": {
        "desc": "Problem response",
        "name": "PRR_PC5",
        "segments": {
            "desc": "Problem response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "PROBLEM",
                            "desc": "Problem",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PRB",
                                    "desc": "Problem detail",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "PROBLEM_ROLE",
                                    "desc": "Problem role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM_PATHWAY",
                                    "desc": "Problem pathway",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "PTH",
                                            "desc": "Pathway",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM_OBSERVATION",
                                    "desc": "Problem observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GOAL",
                                    "desc": "Goal",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "GOL",
                                            "desc": "Goal detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "GOAL_ROLE",
                                            "desc": "Goal role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "GOAL_OBSERVATION",
                                            "desc": "Goal observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                },
                                {
                                    "name": "ORDER",
                                    "desc": "Order",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ORC",
                                            "desc": "Common order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ORDER_DETAIL",
                                            "desc": "Order detail",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "OBR",
                                                    "desc": "Observation request segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "ORDER_OBSERVATION",
                                                    "desc": "Order observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "PTR_PCF": {
        "desc": "Pathway (problem-oriented) query response",
        "name": "PTR_PCF",
        "segments": {
            "desc": "Pathway (problem-oriented) query response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "PATHWAY",
                            "desc": "Pathway",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PTH",
                                    "desc": "Pathway",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "VAR",
                                    "desc": "Variance",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "PATHWAY_ROLE",
                                    "desc": "Pathway role",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "ROL",
                                            "desc": "Role",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PROBLEM",
                                    "desc": "Problem",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "PRB",
                                            "desc": "Problem detail",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "VAR",
                                            "desc": "Variance",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "PROBLEM_ROLE",
                                            "desc": "Problem role",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ROL",
                                                    "desc": "Role",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "PROBLEM_OBSERVATION",
                                            "desc": "Problem observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        },
                                        {
                                            "name": "GOAL",
                                            "desc": "Goal",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "GOL",
                                                    "desc": "Goal detail",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "VAR",
                                                    "desc": "Variance",
                                                    "min": 0,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "GOAL_ROLE",
                                                    "desc": "Goal role",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "ROL",
                                                            "desc": "Role",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                },
                                                {
                                                    "name": "GOAL_OBSERVATION",
                                                    "desc": "Goal observation",
                                                    "min": 0,
                                                    "max": 0,
                                                    "children": [
                                                        {
                                                            "name": "OBX",
                                                            "desc": "Observation segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        },
                                        {
                                            "name": "ORDER",
                                            "desc": "Order",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "ORC",
                                                    "desc": "Common order segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "ORDER_DETAIL",
                                                    "desc": "Order detail",
                                                    "min": 0,
                                                    "max": 1,
                                                    "children": [
                                                        {
                                                            "name": "OBR",
                                                            "desc": "Observation request segment",
                                                            "min": 1,
                                                            "max": 1
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "VAR",
                                                            "desc": "Variance",
                                                            "min": 0,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "ORDER_OBSERVATION",
                                                            "desc": "Order observation",
                                                            "min": 0,
                                                            "max": 0,
                                                            "children": [
                                                                {
                                                                    "name": "OBX",
                                                                    "desc": "Observation segment",
                                                                    "min": 1,
                                                                    "max": 1
                                                                },
                                                                {
                                                                    "name": "NTE",
                                                                    "desc": "Notes and comments segment",
                                                                    "min": 0,
                                                                    "max": 0
                                                                },
                                                                {
                                                                    "name": "VAR",
                                                                    "desc": "Variance",
                                                                    "min": 0,
                                                                    "max": 0
                                                                }
                                                            ]
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "QRY_A19": {
        "desc": "Patient query",
        "name": "QRY_A19",
        "segments": {
            "desc": "Patient query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "QRY_PC4": {
        "desc": "Problem query",
        "name": "QRY_PC4",
        "segments": {
            "desc": "Problem query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "QRY_PC9": {
        "desc": "Goal query",
        "name": "QRY_PC9",
        "segments": {
            "desc": "Goal query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "QRY_PCE": {
        "desc": "Pathway (problem-oriented) query",
        "name": "QRY_PCE",
        "segments": {
            "desc": "Pathway (problem-oriented) query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "QRY_PCK": {
        "desc": "Pathway (goal-oriented) query",
        "name": "QRY_PCK",
        "segments": {
            "desc": "Pathway (goal-oriented) query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "QRY_Q01": {
        "desc": "Query sent for immediate response",
        "name": "QRY_Q01",
        "segments": {
            "desc": "Query sent for immediate response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "QRY_R02": {
        "desc": "Query for results of observation",
        "name": "QRY_R02",
        "segments": {
            "desc": "Query for results of observation",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 1,
                    "max": 1
                }
            ]
        }
    },
    "QRY_T12": {
        "desc": "Document query",
        "name": "QRY_T12",
        "segments": {
            "desc": "Document query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "RAR_RAR": {
        "desc": "Pharmacy/treatment administration information",
        "name": "RAR_RAR",
        "segments": {
            "desc": "Pharmacy/treatment administration information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DEFINITION",
                    "desc": "Definition",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "QRD",
                            "desc": "Query definition segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "QRF",
                            "desc": "Query filter segment",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ENCODING",
                                    "desc": "Encoding",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXE",
                                            "desc": "Pharmacy encoded order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "RXA",
                                    "desc": "Pharmacy administration segment",
                                    "min": 1,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "RAS_O01": {
        "desc": "Pharmacy/treatment administration message",
        "name": "RAS_O01",
        "segments": {
            "desc": "Pharmacy/treatment administration message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "ORDER",
                    "desc": "Order",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ORDER_DETAIL",
                            "desc": "Order detail",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "RXO",
                                    "desc": "Pharmacy prescription order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL_SUPPLEMENT",
                                    "desc": "Order detail supplement",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "COMPONENTS",
                                            "desc": "Components",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "RXC",
                                                    "desc": "Pharmacy component order segment",
                                                    "min": 1,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ENCODING",
                            "desc": "Encoding",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "RXE",
                                    "desc": "Pharmacy encoded order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "RXR",
                                    "desc": "Pharmacy route segment",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "RXC",
                                    "desc": "Pharmacy component order segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RXA",
                            "desc": "Pharmacy administration segment",
                            "min": 1,
                            "max": 0
                        },
                        {
                            "name": "RXR",
                            "desc": "Pharmacy route segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "CTI",
                            "desc": "Clinical trial identification",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "RAS_O02": {
        "desc": "Pharmacy/treatment administration message",
        "name": "RAS_O02",
        "segments": {
            "desc": "Pharmacy/treatment administration message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "RCI_I05": {
        "desc": "Return clinical information",
        "name": "RCI_I05",
        "segments": {
            "desc": "Return clinical information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "OBSERVATION",
                    "desc": "Observation",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "RESULTS",
                            "desc": "Results",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RCL_I06": {
        "desc": "Return clinical information",
        "name": "RCL_I06",
        "segments": {
            "desc": "Return clinical information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DSP",
                    "desc": "Display data segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "RDE_O01": {
        "desc": "Pharmacy/treatment encoded order message",
        "name": "RDE_O01",
        "segments": {
            "desc": "Pharmacy/treatment encoded order message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "ORDER",
                    "desc": "Order",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ORDER_DETAIL",
                            "desc": "Order detail",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "RXO",
                                    "desc": "Pharmacy prescription order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "RXR",
                                    "desc": "Pharmacy route segment",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "COMPONENT",
                                    "desc": "Component",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "RXE",
                            "desc": "Pharmacy encoded order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "RXR",
                            "desc": "Pharmacy route segment",
                            "min": 1,
                            "max": 0
                        },
                        {
                            "name": "RXC",
                            "desc": "Pharmacy component order segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "CTI",
                            "desc": "Clinical trial identification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "RDO_O01": {
        "desc": "Rdo message",
        "name": "RDO_O01",
        "segments": {
            "desc": "Rdo message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "ORDER",
                    "desc": "Order",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ORDER_DETAIL",
                            "desc": "Order detail",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "RXO",
                                    "desc": "Pharmacy prescription order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                },
                                {
                                    "name": "RXR",
                                    "desc": "Pharmacy route segment",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "COMPONENT",
                                    "desc": "Component",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "OBSERVATION",
                                    "desc": "Observation",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "OBX",
                                            "desc": "Observation segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "BLG",
                            "desc": "Billing segment",
                            "min": 0,
                            "max": 1
                        }
                    ]
                }
            ]
        }
    },
    "RDR_RDR": {
        "desc": "Pharmacy/treatment dispense information",
        "name": "RDR_RDR",
        "segments": {
            "desc": "Pharmacy/treatment dispense information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DEFINITION",
                    "desc": "Definition",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "QRD",
                            "desc": "Query definition segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "QRF",
                            "desc": "Query filter segment",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ENCODING",
                                    "desc": "Encoding",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXE",
                                            "desc": "Pharmacy encoded order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "DISPENSE",
                                    "desc": "Dispense",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "RXD",
                                            "desc": "Pharmacy dispense segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "RDS_O01": {
        "desc": "Pharmacy/treatment dispense information",
        "name": "RDS_O01",
        "segments": {
            "desc": "Pharmacy/treatment dispense information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PD1",
                            "desc": "Patient demographic",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "AL1",
                            "desc": "Patient allergy information",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT_VISIT",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "ORDER",
                    "desc": "Order",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "ORDER_DETAIL",
                            "desc": "Order detail",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "RXO",
                                    "desc": "Pharmacy prescription order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL_SUPPLEMENT",
                                    "desc": "Order detail supplement",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "COMPONENT",
                                            "desc": "Component",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "RXC",
                                                    "desc": "Pharmacy component order segment",
                                                    "min": 1,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ENCODING",
                            "desc": "Encoding",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "RXE",
                                    "desc": "Pharmacy encoded order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "RXR",
                                    "desc": "Pharmacy route segment",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "RXC",
                                    "desc": "Pharmacy component order segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RXD",
                            "desc": "Pharmacy dispense segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "RXR",
                            "desc": "Pharmacy route segment",
                            "min": 1,
                            "max": 0
                        },
                        {
                            "name": "RXC",
                            "desc": "Pharmacy component order segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "REF_I12": {
        "desc": "Patient referral",
        "name": "REF_I12",
        "segments": {
            "desc": "Patient referral",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "REF_I13": {
        "desc": "Modify patient referral",
        "name": "REF_I13",
        "segments": {
            "desc": "Modify patient referral",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "REF_I14": {
        "desc": "Cancel patient referral",
        "name": "REF_I14",
        "segments": {
            "desc": "Cancel patient referral",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "REF_I15": {
        "desc": "Request patient referral status",
        "name": "REF_I15",
        "segments": {
            "desc": "Request patient referral status",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RER_RER": {
        "desc": "Pharmacy/treatment encoded order information",
        "name": "RER_RER",
        "segments": {
            "desc": "Pharmacy/treatment encoded order information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DEFINITION",
                    "desc": "Definition",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "QRD",
                            "desc": "Query definition segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "QRF",
                            "desc": "Query filter segment",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "RXE",
                                    "desc": "Pharmacy encoded order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "RXR",
                                    "desc": "Pharmacy route segment",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "RXC",
                                    "desc": "Pharmacy component order segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "RGR_RGR": {
        "desc": "Pharmacy/treatment dose information",
        "name": "RGR_RGR",
        "segments": {
            "desc": "Pharmacy/treatment dose information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DEFINITION",
                    "desc": "Definition",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "QRD",
                            "desc": "Query definition segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "QRF",
                            "desc": "Query filter segment",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ENCODING",
                                    "desc": "Encoding",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXE",
                                            "desc": "Pharmacy encoded order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "RXG",
                                    "desc": "Pharmacy give segment",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "RXR",
                                    "desc": "Pharmacy route segment",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "RXC",
                                    "desc": "Pharmacy component order segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "RGV_O01": {
        "desc": "Pharmacy/treatment give message",
        "name": "RGV_O01",
        "segments": {
            "desc": "Pharmacy/treatment give message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "XX",
                            "desc": "Xx",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AL1",
                                    "desc": "Patient allergy information",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "PATIENT_VISIT",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "PV1",
                                            "desc": "Patient visit",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "PV2",
                                            "desc": "Patient visit - additional information",
                                            "min": 0,
                                            "max": 1
                                        }
                                    ]
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL",
                                    "desc": "Order detail",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXO",
                                            "desc": "Pharmacy prescription order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "ORDER_DETAIL_SUPPLEMENT",
                                            "desc": "Order detail supplement",
                                            "min": 0,
                                            "max": 1,
                                            "children": [
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 1,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "RXR",
                                                    "desc": "Pharmacy route segment",
                                                    "min": 1,
                                                    "max": 0
                                                },
                                                {
                                                    "name": "COMPONENTS",
                                                    "desc": "Components",
                                                    "min": 0,
                                                    "max": 1,
                                                    "children": [
                                                        {
                                                            "name": "RXC",
                                                            "desc": "Pharmacy component order segment",
                                                            "min": 1,
                                                            "max": 0
                                                        },
                                                        {
                                                            "name": "NTE",
                                                            "desc": "Notes and comments segment",
                                                            "min": 0,
                                                            "max": 0
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    ]
                                },
                                {
                                    "name": "ENCODING",
                                    "desc": "Encoding",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXE",
                                            "desc": "Pharmacy encoded order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GIVE",
                                    "desc": "Give",
                                    "min": 1,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "RXG",
                                            "desc": "Pharmacy give segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "OBSERVATION",
                                            "desc": "Observation",
                                            "min": 0,
                                            "max": 0,
                                            "children": [
                                                {
                                                    "name": "OBX",
                                                    "desc": "Observation segment",
                                                    "min": 1,
                                                    "max": 1
                                                },
                                                {
                                                    "name": "NTE",
                                                    "desc": "Notes and comments segment",
                                                    "min": 0,
                                                    "max": 0
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "ROR_ROR": {
        "desc": "Pharmacy prescription order query response",
        "name": "ROR_ROR",
        "segments": {
            "desc": "Pharmacy prescription order query response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DEFINITION",
                    "desc": "Definition",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "QRD",
                            "desc": "Query definition segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "QRF",
                            "desc": "Query filter segment",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "RXO",
                                    "desc": "Pharmacy prescription order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "RXR",
                                    "desc": "Pharmacy route segment",
                                    "min": 1,
                                    "max": 0
                                },
                                {
                                    "name": "RXC",
                                    "desc": "Pharmacy component order segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "RPA_I08": {
        "desc": "Return patient authorization",
        "name": "RPA_I08",
        "segments": {
            "desc": "Return patient authorization",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "OBSERVATION",
                    "desc": "Observation",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "RESULTS",
                            "desc": "Results",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "RPA_I09": {
        "desc": "Return patient authorization",
        "name": "RPA_I09",
        "segments": {
            "desc": "Return patient authorization",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "RPA_I10": {
        "desc": "Return patient authorization",
        "name": "RPA_I10",
        "segments": {
            "desc": "Return patient authorization",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "RPA_I11": {
        "desc": "Return patient authorization",
        "name": "RPA_I11",
        "segments": {
            "desc": "Return patient authorization",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    },
    "RPI_I01": {
        "desc": "Return patient information",
        "name": "RPI_I01",
        "segments": {
            "desc": "Return patient information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RPI_I04": {
        "desc": "Return patient information",
        "name": "RPI_I04",
        "segments": {
            "desc": "Return patient information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RPL_I02": {
        "desc": "Return patient display list",
        "name": "RPL_I02",
        "segments": {
            "desc": "Return patient display list",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DSP",
                    "desc": "Display data segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "RPR_I03": {
        "desc": "Return patient list",
        "name": "RPR_I03",
        "segments": {
            "desc": "Return patient list",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQA_I08": {
        "desc": "Request for treatment authorization information",
        "name": "RQA_I08",
        "segments": {
            "desc": "Request for treatment authorization information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "OBSERVATION",
                    "desc": "Observation",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "RESULTS",
                            "desc": "Results",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQA_I09": {
        "desc": "Request for modification to an authorization",
        "name": "RQA_I09",
        "segments": {
            "desc": "Request for modification to an authorization",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQA_I10": {
        "desc": "Request for resubmission of an authorization",
        "name": "RQA_I10",
        "segments": {
            "desc": "Request for resubmission of an authorization",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQA_I11": {
        "desc": "Request for cancellation of an authorization",
        "name": "RQA_I11",
        "segments": {
            "desc": "Request for cancellation of an authorization",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQC_I05": {
        "desc": "Request for patient clinical information",
        "name": "RQC_I05",
        "segments": {
            "desc": "Request for patient clinical information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQC_I06": {
        "desc": "Request/receipt of clinical data listing",
        "name": "RQC_I06",
        "segments": {
            "desc": "Request/receipt of clinical data listing",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQI_I01": {
        "desc": "Request for insurance information",
        "name": "RQI_I01",
        "segments": {
            "desc": "Request for insurance information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PRDCTD",
                    "desc": "Prdctd",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQI_I02": {
        "desc": "Request/receipt of patient selection display list",
        "name": "RQI_I02",
        "segments": {
            "desc": "Request/receipt of patient selection display list",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQI_I03": {
        "desc": "Request/receipt of patient selection list",
        "name": "RQI_I03",
        "segments": {
            "desc": "Request/receipt of patient selection list",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GUARANTOR_INSURANCE",
                    "desc": "Guarantor insurance",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "GT1",
                            "desc": "Guarantor",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "INSURANCE",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "IN1",
                                    "desc": "Insurance",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "IN2",
                                    "desc": "Insurance additional info",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "IN3",
                                    "desc": "Insurance additional info - certification",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQP_I04": {
        "desc": "Request patient demographics",
        "name": "RQP_I04",
        "segments": {
            "desc": "Request patient demographics",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "GT1",
                    "desc": "Guarantor",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RQQ_Q01": {
        "desc": "Event replay query",
        "name": "RQQ_Q01",
        "segments": {
            "desc": "Event replay query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERQ",
                    "desc": "Event replay query segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "RRA_O02": {
        "desc": "Pharmacy/treatment administration acknowledgment message",
        "name": "RRA_O02",
        "segments": {
            "desc": "Pharmacy/treatment administration acknowledgment message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ADMINISTRATION",
                                    "desc": "Administration",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "RXA",
                                            "desc": "Pharmacy administration segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 1
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "RRD_O02": {
        "desc": "Pharmacy/treatment dispense acknowledgment message",
        "name": "RRD_O02",
        "segments": {
            "desc": "Pharmacy/treatment dispense acknowledgment message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "RESPONSE",
                            "desc": "Response",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "DISPENSE",
                                    "desc": "Dispense",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXD",
                                            "desc": "Pharmacy dispense segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "RRE_O01": {
        "desc": "Pharmacy/treatment encoded order acknowledgment message",
        "name": "RRE_O01",
        "segments": {
            "desc": "Pharmacy/treatment encoded order acknowledgment message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ENCODING",
                                    "desc": "Encoding",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXE",
                                            "desc": "Pharmacy encoded order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "RRG_O02": {
        "desc": "Pharmacy/treatment give acknowledgment message",
        "name": "RRG_O02",
        "segments": {
            "desc": "Pharmacy/treatment give acknowledgment message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "GIVE",
                                    "desc": "Give",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXG",
                                            "desc": "Pharmacy give segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "RRI_I12": {
        "desc": "Return referral information",
        "name": "RRI_I12",
        "segments": {
            "desc": "Return referral information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RRI_I13": {
        "desc": "Return referral information",
        "name": "RRI_I13",
        "segments": {
            "desc": "Return referral information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RRI_I14": {
        "desc": "Return referral information",
        "name": "RRI_I14",
        "segments": {
            "desc": "Return referral information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RRI_I15": {
        "desc": "Return referral information",
        "name": "RRI_I15",
        "segments": {
            "desc": "Return referral information",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "RF1",
                    "desc": "Referral information segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "AUTHORIZATION",
                    "desc": "Authorization",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "AUT",
                            "desc": "Authorization information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "PROVIDER",
                    "desc": "Provider",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PRD",
                            "desc": "Provider data",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "CTD",
                            "desc": "Contact data",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ACC",
                    "desc": "Accident",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DG1",
                    "desc": "Diagnosis",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "DRG",
                    "desc": "Diagnosis related group",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "AL1",
                    "desc": "Patient allergy information",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PROCEDURE",
                    "desc": "Procedure",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PR1",
                            "desc": "Procedures",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "AUTHORIZATION",
                            "desc": "Authorization",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "AUT",
                                    "desc": "Authorization information",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "CTD",
                                    "desc": "Contact data",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "RESULTS",
                    "desc": "Results",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "OBR",
                            "desc": "Observation request segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "VISIT",
                    "desc": "Visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                }
            ]
        }
    },
    "RRO_O02": {
        "desc": "Rro message",
        "name": "RRO_O02",
        "segments": {
            "desc": "Rro message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "RESPONSE",
                    "desc": "Response",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "ORDER",
                            "desc": "Order",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "ORC",
                                    "desc": "Common order segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "ORDER_DETAIL",
                                    "desc": "Order detail",
                                    "min": 0,
                                    "max": 1,
                                    "children": [
                                        {
                                            "name": "RXO",
                                            "desc": "Pharmacy prescription order segment",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXR",
                                            "desc": "Pharmacy route segment",
                                            "min": 1,
                                            "max": 0
                                        },
                                        {
                                            "name": "RXC",
                                            "desc": "Pharmacy component order segment",
                                            "min": 0,
                                            "max": 0
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S12": {
        "desc": "Notification of new appointment booking",
        "name": "SIU_S12",
        "segments": {
            "desc": "Notification of new appointment booking",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S13": {
        "desc": "Notification of appointment rescheduling",
        "name": "SIU_S13",
        "segments": {
            "desc": "Notification of appointment rescheduling",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S14": {
        "desc": "Notification of appointment modification",
        "name": "SIU_S14",
        "segments": {
            "desc": "Notification of appointment modification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S15": {
        "desc": "Notification of appointment cancellation",
        "name": "SIU_S15",
        "segments": {
            "desc": "Notification of appointment cancellation",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S16": {
        "desc": "Notification of appointment discontinuation",
        "name": "SIU_S16",
        "segments": {
            "desc": "Notification of appointment discontinuation",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATIONL_RESOURCE",
                            "desc": "Locationl resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S17": {
        "desc": "Notification of appointment deletion",
        "name": "SIU_S17",
        "segments": {
            "desc": "Notification of appointment deletion",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S18": {
        "desc": "Notification of addition of service/resource on appointment",
        "name": "SIU_S18",
        "segments": {
            "desc": "Notification of addition of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S19": {
        "desc": "Notification of modification of service/resource on appointment",
        "name": "SIU_S19",
        "segments": {
            "desc": "Notification of modification of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S20": {
        "desc": "Notification of cancellation of service/resource on appointment",
        "name": "SIU_S20",
        "segments": {
            "desc": "Notification of cancellation of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S21": {
        "desc": "Notification of discontinuation of service/resource on appointment",
        "name": "SIU_S21",
        "segments": {
            "desc": "Notification of discontinuation of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S22": {
        "desc": "Notification of deletion of service/resource on appointment",
        "name": "SIU_S22",
        "segments": {
            "desc": "Notification of deletion of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S23": {
        "desc": "Notification of blocked schedule time slot(s)",
        "name": "SIU_S23",
        "segments": {
            "desc": "Notification of blocked schedule time slot(s)",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S24": {
        "desc": "Notification of opened (\"unblocked\") schedule time slot(s)",
        "name": "SIU_S24",
        "segments": {
            "desc": "Notification of opened (\"unblocked\") schedule time slot(s)",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SIU_S26": {
        "desc": "Notification that patient did not show up for schedule appointment",
        "name": "SIU_S26",
        "segments": {
            "desc": "Notification that patient did not show up for schedule appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCH",
                    "desc": "Schedule activity information",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SPQ_Q01": {
        "desc": "Stored procedure request",
        "name": "SPQ_Q01",
        "segments": {
            "desc": "Stored procedure request",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SPR",
                    "desc": "Stored procedure request definition",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RDF",
                    "desc": "Table row definition",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "SQM_S25": {
        "desc": "Schedule query message and response",
        "name": "SQM_S25",
        "segments": {
            "desc": "Schedule query message and response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "REQUEST",
                    "desc": "Request",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "ARQ",
                            "desc": "Appointment request",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "APR",
                            "desc": "Appointment preferences",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "APR",
                                            "desc": "Appointment preferences",
                                            "min": 0,
                                            "max": 1
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "APR",
                                            "desc": "Appointment preferences",
                                            "min": 0,
                                            "max": 1
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "APR",
                                            "desc": "Appointment preferences",
                                            "min": 0,
                                            "max": 1
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "APR",
                                            "desc": "Appointment preferences",
                                            "min": 0,
                                            "max": 1
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "SQR_S25": {
        "desc": "Schedule query message and response",
        "name": "SQR_S25",
        "segments": {
            "desc": "Schedule query message and response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QAK",
                    "desc": "Query acknowledgement",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 1,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "SRM_S01": {
        "desc": "Request new appointment booking",
        "name": "SRM_S01",
        "segments": {
            "desc": "Request new appointment booking",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S02": {
        "desc": "Request appointment rescheduling",
        "name": "SRM_S02",
        "segments": {
            "desc": "Request appointment rescheduling",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S03": {
        "desc": "Request appointment modification",
        "name": "SRM_S03",
        "segments": {
            "desc": "Request appointment modification",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S04": {
        "desc": "Request appointment cancellation",
        "name": "SRM_S04",
        "segments": {
            "desc": "Request appointment cancellation",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S05": {
        "desc": "Request appointment discontinuation",
        "name": "SRM_S05",
        "segments": {
            "desc": "Request appointment discontinuation",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S06": {
        "desc": "Request appointment deletion",
        "name": "SRM_S06",
        "segments": {
            "desc": "Request appointment deletion",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S07": {
        "desc": "Request addition of service/resource on appointment",
        "name": "SRM_S07",
        "segments": {
            "desc": "Request addition of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S08": {
        "desc": "Request modification of service/resource on appointment",
        "name": "SRM_S08",
        "segments": {
            "desc": "Request modification of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S09": {
        "desc": "Request cancellation of service/resource on appointment",
        "name": "SRM_S09",
        "segments": {
            "desc": "Request cancellation of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S10": {
        "desc": "Request discontinuation of service/resource on appointment",
        "name": "SRM_S10",
        "segments": {
            "desc": "Request discontinuation of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRM_S11": {
        "desc": "Request deletion of service/resource on appointment",
        "name": "SRM_S11",
        "segments": {
            "desc": "Request deletion of service/resource on appointment",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ARQ",
                    "desc": "Appointment request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "APR",
                    "desc": "Appointment preferences",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NTE",
                    "desc": "Notes and comments segment",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBX",
                            "desc": "Observation segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "DG1",
                            "desc": "Diagnosis",
                            "min": 0,
                            "max": 0
                        }
                    ]
                },
                {
                    "name": "RESOURCES",
                    "desc": "Resources",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "RGS",
                            "desc": "Resource group",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "SERVICE",
                            "desc": "Service",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIS",
                                    "desc": "Appointment information - service",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "GENERAL_RESOURCE",
                            "desc": "General resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIG",
                                    "desc": "Appointment information - general resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "LOCATION_RESOURCE",
                            "desc": "Location resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIL",
                                    "desc": "Appointment information - location resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "PERSONNEL_RESOURCE",
                            "desc": "Personnel resource",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "AIP",
                                    "desc": "Appointment information - personnel resource",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "APR",
                                    "desc": "Appointment preferences",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S01": {
        "desc": "Scheduled request response",
        "name": "SRR_S01",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S02": {
        "desc": "Scheduled request response",
        "name": "SRR_S02",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S03": {
        "desc": "Scheduled request response",
        "name": "SRR_S03",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S04": {
        "desc": "Scheduled request response",
        "name": "SRR_S04",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S05": {
        "desc": "Scheduled request response",
        "name": "SRR_S05",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S06": {
        "desc": "Scheduled request response",
        "name": "SRR_S06",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S07": {
        "desc": "Scheduled request response",
        "name": "SRR_S07",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S08": {
        "desc": "Scheduled request response",
        "name": "SRR_S08",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S09": {
        "desc": "Scheduled request response",
        "name": "SRR_S09",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S10": {
        "desc": "Scheduled request response",
        "name": "SRR_S10",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SRR_S11": {
        "desc": "Scheduled request response",
        "name": "SRR_S11",
        "segments": {
            "desc": "Scheduled request response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "SCHEDULE",
                    "desc": "Schedule",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "SCH",
                            "desc": "Schedule activity information",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NTE",
                            "desc": "Notes and comments segment",
                            "min": 0,
                            "max": 0
                        },
                        {
                            "name": "PATIENT",
                            "desc": "Patient",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PID",
                                    "desc": "Patient identification",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PV1",
                                    "desc": "Patient visit",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "PV2",
                                    "desc": "Patient visit - additional information",
                                    "min": 0,
                                    "max": 1
                                },
                                {
                                    "name": "DG1",
                                    "desc": "Diagnosis",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        },
                        {
                            "name": "RESOURCES",
                            "desc": "Resources",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "RGS",
                                    "desc": "Resource group",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "SERVICE",
                                    "desc": "Service",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIS",
                                            "desc": "Appointment information - service",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "GENERAL_RESOURCE",
                                    "desc": "General resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIG",
                                            "desc": "Appointment information - general resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "LOCATION_RESOURCE",
                                    "desc": "Location resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIL",
                                            "desc": "Appointment information - location resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                },
                                {
                                    "name": "PERSONNEL_RESOURCE",
                                    "desc": "Personnel resource",
                                    "min": 0,
                                    "max": 0,
                                    "children": [
                                        {
                                            "name": "AIP",
                                            "desc": "Appointment information - personnel resource",
                                            "min": 1,
                                            "max": 1
                                        },
                                        {
                                            "name": "NTE",
                                            "desc": "Notes and comments segment",
                                            "min": 0,
                                            "max": 0
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "SUR_P09": {
        "desc": "Summary product experience report",
        "name": "SUR_P09",
        "segments": {
            "desc": "Summary product experience report",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "FACILITY",
                    "desc": "Facility",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "FAC",
                            "desc": "Facility",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PRODUCT",
                            "desc": "Product",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "PSH",
                                    "desc": "Product summary header",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PDC",
                                    "desc": "Product detail country",
                                    "min": 1,
                                    "max": 1
                                }
                            ]
                        },
                        {
                            "name": "PSH",
                            "desc": "Product summary header",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "FACILITY_DETAIL",
                            "desc": "Facility detail",
                            "min": 1,
                            "max": 0,
                            "children": [
                                {
                                    "name": "FAC",
                                    "desc": "Facility",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "PDC",
                                    "desc": "Product detail country",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 1,
                                    "max": 1
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "TBR_Q01": {
        "desc": "Tabular data response",
        "name": "TBR_Q01",
        "segments": {
            "desc": "Tabular data response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "ERR",
                    "desc": "Error segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "QAK",
                    "desc": "Query acknowledgement",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RDF",
                    "desc": "Table row definition",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RDT",
                    "desc": "Table row data",
                    "min": 1,
                    "max": 0
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "UDM_Q05": {
        "desc": "Unsolicited display update message",
        "name": "UDM_Q05",
        "segments": {
            "desc": "Unsolicited display update message",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "URD",
                    "desc": "Results/update definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "URS",
                    "desc": "Unsolicited selection segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DSP",
                    "desc": "Display data segment",
                    "min": 1,
                    "max": 0
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "VQQ_Q01": {
        "desc": "Virtual table query",
        "name": "VQQ_Q01",
        "segments": {
            "desc": "Virtual table query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "VTQ",
                    "desc": "Virtual table query request",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "RDF",
                    "desc": "Table row definition",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "DSC",
                    "desc": "Continuation pointer segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "VXQ_V01": {
        "desc": "Query for vaccination record",
        "name": "VXQ_V01",
        "segments": {
            "desc": "Query for vaccination record",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                }
            ]
        }
    },
    "VXR_V03": {
        "desc": "Vaccination record response",
        "name": "VXR_V03",
        "segments": {
            "desc": "Vaccination record response",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT_VISIT",
                    "desc": "Patient visit",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ORDER",
                    "desc": "Order",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "RXA",
                            "desc": "Pharmacy administration segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "RXR",
                            "desc": "Pharmacy route segment",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "VXU_V04": {
        "desc": "Unsolicited vaccination record update",
        "name": "VXU_V04",
        "segments": {
            "desc": "Unsolicited vaccination record update",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PID",
                    "desc": "Patient identification",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "PD1",
                    "desc": "Patient demographic",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "NK1",
                    "desc": "Next of kin",
                    "min": 0,
                    "max": 0
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 0,
                    "max": 1,
                    "children": [
                        {
                            "name": "PV1",
                            "desc": "Patient visit",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "PV2",
                            "desc": "Patient visit - additional information",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "INSURANCE",
                    "desc": "Insurance",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "IN1",
                            "desc": "Insurance",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "IN2",
                            "desc": "Insurance additional info",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "IN3",
                            "desc": "Insurance additional info - certification",
                            "min": 0,
                            "max": 1
                        }
                    ]
                },
                {
                    "name": "ORDER",
                    "desc": "Order",
                    "min": 0,
                    "max": 0,
                    "children": [
                        {
                            "name": "ORC",
                            "desc": "Common order segment",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "RXA",
                            "desc": "Pharmacy administration segment",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "RXR",
                            "desc": "Pharmacy route segment",
                            "min": 0,
                            "max": 1
                        },
                        {
                            "name": "OBSERVATION",
                            "desc": "Observation",
                            "min": 0,
                            "max": 0,
                            "children": [
                                {
                                    "name": "OBX",
                                    "desc": "Observation segment",
                                    "min": 1,
                                    "max": 1
                                },
                                {
                                    "name": "NTE",
                                    "desc": "Notes and comments segment",
                                    "min": 0,
                                    "max": 0
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    },
    "VXX_V02": {
        "desc": "Response to vaccination query",
        "name": "VXX_V02",
        "segments": {
            "desc": "Response to vaccination query",
            "segments": [
                {
                    "name": "MSH",
                    "desc": "Message header segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "MSA",
                    "desc": "Message acknowledgement segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRD",
                    "desc": "Query definition segment",
                    "min": 1,
                    "max": 1
                },
                {
                    "name": "QRF",
                    "desc": "Query filter segment",
                    "min": 0,
                    "max": 1
                },
                {
                    "name": "PATIENT",
                    "desc": "Patient",
                    "min": 1,
                    "max": 0,
                    "children": [
                        {
                            "name": "PID",
                            "desc": "Patient identification",
                            "min": 1,
                            "max": 1
                        },
                        {
                            "name": "NK1",
                            "desc": "Next of kin",
                            "min": 0,
                            "max": 0
                        }
                    ]
                }
            ]
        }
    }
};

module.exports = messages;
